@@ -1,4 +1,4 @@
-/* $Id: milter-greylist.c,v 1.222 2009/09/09 12:19:17 manu Exp $ */
+/* $Id: milter-greylist.c,v 1.230 2010/04/18 04:03:56 manu Exp $ */
 
 /*
  * Copyright (c) 2004-2007 Emmanuel Dreyfus
@@ -34,7 +34,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID  
-__RCSID("$Id: milter-greylist.c,v 1.222 2009/09/09 12:19:17 manu Exp $");
+__RCSID("$Id: milter-greylist.c,v 1.230 2010/04/18 04:03:56 manu Exp $");
 #endif
 #endif
 
@@ -111,6 +111,7 @@ static int check_drac(char *dotted_ip);
 #include "p0f.h"
 #endif
 #include "macro.h"
+#include "ratelimit.h"
 
 #ifdef USE_DMALLOC
 #include <dmalloc.h> 
@@ -317,18 +318,24 @@ static sfsistat
 tarpit_reentry(priv)
 	struct mlfi_priv *priv;
 {
+	struct tuple_fields tuple;
+
 	sfsistat stat = SMFIS_CONTINUE;
 
 	if (!(priv->priv_sr.sr_whitelist & EXF_TARPIT))
 		return stat;
 
 	if (priv->priv_sr.sr_whitelist & EXF_WHITELIST) {
-		struct rcpt *rcpt;
 
-		rcpt = priv->priv_rcpt.lh_first;
-		pending_update(SA(&priv->priv_addr), priv->priv_addrlen,
-			       priv->priv_from, rcpt->r_addr,
-			       priv->priv_sr.sr_autowhite, TU_AUTOWHITE);
+		tuple.sa = SA(&priv->priv_addr);
+		tuple.salen = priv->priv_addrlen;
+		tuple.from = priv->priv_from;
+		tuple.rcpt = priv->priv_rcpt.lh_first->r_addr;
+		tuple.autowhite = 0;
+		tuple.updatetype = TU_AUTOWHITE;
+
+		mg_tuple_update(&tuple);
+
 	} else if (priv->priv_sr.sr_whitelist & EXF_GREYLIST) {
 		stat = SMFIS_TEMPFAIL;
 	}
@@ -584,7 +591,7 @@ real_envrcpt(ctx, envrcpt)
 	char *greylist;
 	char addrstr[IPADDRSTRLEN];
 	char rcpt[ADDRLEN + 1];
-	int save_nolog;
+	long long save_nolog;
 	struct tuple_fields tuple;
 	time_t sleep_duration = 0;
 	/*
@@ -715,8 +722,8 @@ real_envrcpt(ctx, envrcpt)
 	tuple.gldelay = priv->priv_sr.sr_delay;
 	tuple.autowhite = priv->priv_sr.sr_autowhite;
 
-	switch(mg_tuple_check(tuple)) {
-	case T_AUTOWHITE:			/* autowhite listed */
+	switch(mg_tuple_check(&tuple)) {
+	case T_AUTOWHITE:		/* autowhite listed */
 		priv->priv_sr.sr_elapsed = 0;
 		priv->priv_sr.sr_whitelist = EXF_WHITELIST | EXF_AUTO;
 		goto exit_accept;
@@ -744,11 +751,10 @@ real_envrcpt(ctx, envrcpt)
 			priv->priv_total_tarpitted += sleep_duration;
 			sleep(sleep_duration);
 		} else {
-		    if (priv->priv_sr.sr_whitelist & EXF_WHITELIST)
-			pending_update(SA(&priv->priv_addr), priv->priv_addrlen,
-				       priv->priv_from, rcpt,
-				       priv->priv_sr.sr_autowhite,
-				       TU_AUTOWHITE);
+		    if (priv->priv_sr.sr_whitelist & EXF_WHITELIST) {
+			tuple.updatetype = TU_AUTOWHITE;
+			mg_tuple_update(&tuple);
+			}
 		}
 		priv->priv_sr.sr_elapsed = 0;
 		goto exit_accept;
@@ -882,10 +888,11 @@ real_body(ctx, chunk, size)
 		return stat;
 #endif
 
+	priv->priv_msgcount += size;
+
 	/* Avoid copying the whole message to save CPU */
 	if ((priv->priv_msgcount > conf.c_maxpeek) || 
 	    (priv->priv_buflen > conf.c_maxpeek)) {
-		priv->priv_msgcount += size;
 		mg_log(LOG_DEBUG, "ignoring message beyond maxpeek = %d", 
 		    conf.c_maxpeek);
 		return SMFIS_CONTINUE;
@@ -1077,7 +1084,7 @@ real_eom(ctx)
 			tuple.gldelay = priv->priv_sr.sr_delay;
 			tuple.autowhite = priv->priv_sr.sr_autowhite;
 
-			switch(mg_tuple_check(tuple)) {
+			switch(mg_tuple_check(&tuple)) {
 			case T_AUTOWHITE:	/* autowhite listed */
 				priv->priv_sr.sr_whitelist |= 
 				    (EXF_WHITELIST | EXF_AUTO);
@@ -1210,6 +1217,11 @@ passed:
 				    "Sender IP whitelisted by DNSRBL");
 				priv->priv_last_whitelist &= ~EXF_DNSRBL;
 			}
+			if (priv->priv_last_whitelist & EXF_MX) {
+				ADD_REASON(whystr, 
+				    "Sender IP whitelisted by MX");
+				priv->priv_last_whitelist &= ~EXF_MX;
+			}
 			if (priv->priv_last_whitelist & EXF_URLCHECK) {
 				ADD_REASON(whystr, "URL check passed");
 				priv->priv_last_whitelist &= ~EXF_URLCHECK;
@@ -1285,15 +1297,20 @@ real_abort(ctx)
 	SMFICTX *ctx;
 {
 	struct mlfi_priv *priv;
+	struct tuple_fields tuple;
 
 	if ((priv = (struct mlfi_priv *) smfi_getpriv(ctx)) != NULL) {
 		if (priv->priv_sr.sr_whitelist & EXF_TARPIT) {
-			struct rcpt *rcpt;
 
-			rcpt = priv->priv_rcpt.lh_first;
-			pending_update(SA(&priv->priv_addr), priv->priv_addrlen,
-				       priv->priv_from, rcpt->r_addr,
-				       0, TU_TARPIT);
+			tuple.sa = SA(&priv->priv_addr);
+			tuple.salen = priv->priv_addrlen;
+			tuple.from = priv->priv_from;
+			tuple.rcpt = priv->priv_rcpt.lh_first->r_addr;
+			tuple.autowhite = 0;
+			tuple.updatetype = TU_TARPIT;
+
+			mg_tuple_update(&tuple);
+
 			priv->priv_sr.sr_whitelist &= ~EXF_TARPIT;
 		}
 	}
@@ -1602,8 +1619,12 @@ main(argc, argv)
 #endif
 #ifdef USE_DKIM
 	dkimcheck_init();
+#endif
+#ifdef USE_GEOIP
+	geoip_init();
 #endif
 	macro_init();
+	ratelimit_init();
 
 #ifdef USE_FD_POOL
         /* initialize file descriptor pool */
@@ -1634,27 +1655,6 @@ main(argc, argv)
 	cleanup_sock(conf.c_socket);
 	cleanup_pidfile(conf.c_pidfile);
 
-	/*
-	 * Set socket permission
-	 */
-	if (conf.c_socket_mode != -1) {
-		switch(conf.c_socket_mode) {
-		case 666:
-			umask(000);
-			break;
-		case 660:
-			umask(007);
-			break;
-		case 600:
-			umask(077);
-			break;
-		default:
-			mg_log(LOG_ERR, "unexpected socket mode %d",
-			    conf.c_socket_mode);
-			exit(EX_SOFTWARE);
-			break;
-		}
-	}
 	(void)smfi_setconn(conf.c_socket);
 
 	/*
@@ -1679,6 +1679,34 @@ main(argc, argv)
 		exit(EX_UNAVAILABLE);
 	}
 
+	if (smfi_opensocket(1) == MI_FAILURE) {
+		mg_log(LOG_ERR, "%s: failed to open socket: %s",
+		       argv[0], conf.c_socket);
+		exit(EX_UNAVAILABLE);
+	}
+	/*
+	 * Set socket permission
+	 */
+	if (conf.c_socket_mode != -1) {
+		switch(conf.c_socket_mode) {
+		case 0666:
+		case 0660:
+		case 0600:
+			if (chmod(conf.c_socket, conf.c_socket_mode) == -1)
+				mg_log(LOG_ERR,
+				       "failed to change socket mode: %s: %s: O%o",
+				       strerror(errno),
+				       conf.c_socket,
+				       conf.c_socket_mode);
+			break;
+		default:
+			mg_log(LOG_ERR, "unexpected socket mode %d",
+			    conf.c_socket_mode);
+			exit(EX_SOFTWARE);
+			break;
+		}
+	}
+
 	/*
 	 * Turn into a daemon
 	 */
@@ -3073,7 +3101,6 @@ fstring_expand(priv, rcpt, fstring)
 				snprintf(buf, sizeof(buf), "%d", 
 				   priv->priv_sr.sr_acl_line); 
 				mystrncat(&outstr, buf, &outmaxlen);
-				mystrncat(&outstr, "(none)", &outmaxlen);
 			}
 			break;
 		}	
